# Table Creation SQL for ProjectManager Application

#/**
# * Name: project_manager_create.sql [SQL data definition file]
# * Author: R. Joseph Newton
# * Purpose: Provides table definitions, stored procedures and other database
# * structures used in the Project Manager application package.
# * Last Modified: March 29, 2007
# */

# user table creation
create table user (
   username       varchar(32) primary key,
   password       char(32)    not null,
   person_guid    integer     not null
);

# Admin startup user creation
insert into user values('admin', md5('adMin4StArtuP'), 0);

# User create procedure creation
delimiter //
create procedure register_user (out succeeded boolean, in user varchar(32),
 in pass character(32), in person_id integer)
BEGIN
   insert into user values (user, pass, person_id);
   select true;
END
//
delimiter ;

# ticket table creation
create table ticket (
   ticket         char(32)          primary key,
   member         varchar(64)       not null,
   last_connect   integer           not null,
   logon_time     integer           not null
);

# check_ticket procedure creation
delimiter //
create procedure check_ticket (out ticket_count bigint,
 in test_ticket char(32), in login_name varchar(64), in timeout_time integer,
 in time_now integer)
BEGIN
   delete from tickets where last_connect < timeout_time;
   update tickets set last_connect = time_now where ticket = test_ticket;
   select count (ticket) as ticket_count from
      (select ticket from tickets
         where ticket = test_ticket and member = login_name
      ) as matching_tickets;
END
//
delimiter ;

# project_space_guid primary key sequence
CREATE TABLE project_space_guid (id INT NOT NULL);
INSERT INTO project_space_guid VALUES (0);

delimiter //
create procedure increment_project_space_guid (out current integer)
BEGIN
   update project_space_guid set id=last_insert_id(id+1);
   select last_insert_id() into current;
END
//
delimiter ;

# project_space table creation
create table project_space (
   project_space_guid   integer     not null primary key,
   name                 varchar(64) not null,
   description          text
);

# privilege_guid primary key sequence
CREATE TABLE privilege_guid (id INT NOT NULL);
INSERT INTO privilege_guid VALUES (0);

delimiter //
create procedure increment_privilege_guid (out current integer)
BEGIN
   update privilege_guid set id=last_insert_id(id+1);
   select last_insert_id() into current;
END
//
delimiter ;

# privilege table creation
create table privilege (
   privilege_guid    integer     not null    primary key,
   privilege_name    varchar(64) not null    unique,
   description       text
);

# Core privilege creation:
insert into privilege values (1, 'manage', 'Full access to domain');
insert into privilege values (2, 'view', 'Allows viewing of details for domain');
insert into privilege values (3, 'record progress',
 'Allows recording of milestones, deliverables, and project completion');
insert into privilege values (4, 'record expenditures',
 'Allows recording of line-item expenditures');
insert into privilege values (5, 'record allocations',
 'Allows entry of allocation items for domain');
insert into privilege values (6, 'manage accounts',
 'Allows recording of both allocations and expenditures');
insert into privilege values(7, 'add participants',
 'Allows addition of participants to domain');
insert into privilege values(8, 'delegate privileges',
 'Allows delegation of any privileges held to others');
update privilege_guid set id = 8;  # Reflects count of preset privilege records

# project_space table initialization with default project space
insert into project_space values(0, 'default', 'Startup project space.');

# role table creation
create table role (
   role_guid      integer        primary key,
   role_name      varchar(32)    not null,
   scope          enum(
                     'system',
                     'project space',
                     'project',
                     'task')       not null,
   description    text           not null,
   unique (role_name, scope)
);

# Core role creation:
insert into role values (1, 'system administrator', 'system',
 concat('Sets up and repairs system as a whole:',
 ' creates users and project spaces,',
 ' assigns project space administarators'));
insert into role values (2, 'system tech', 'system',
 concat('Provides technical support: analyzes and troubleshoots problems,',
 ' provides feedback, backs up data, may run test transactions'));
insert into role values (3, 'project space administrator', 'project space',
 concat('Manages given project space:',
 ' creates users within space, enables access to users per project,',
 ' initializes projects and assigns leads'));
insert into role values (4, 'project space tech support', 'project space',
 concat('Provides support for functionality within a given project space:',
 ' supports administrator and managers, may provide assistance to team members'));
insert into role values (5, 'lead', 'project', concat('Manages project team:',
 ' maintains schedule and staffing, records transactions, milestones, and ',
 ' deliverables, delegates tasks to team members'));
insert into role values (6, 'co_lead', 'project', concat('Assists project or task',
 ' lead, duties may vary by workgroup, and role may be equivalent to lead in some cases'));
insert into role values (7, 'instigator', 'project', concat('Provides initial impetus for',
 ' initiating project, may provide ongoing advocacy, also'));
insert into role values (8, 'volunteer', 'project', concat('Provides services to project without',
 ' compensation.  Duties may vary, depnding on skills and project'));
insert into role values (9, 'staff participant', 'project space', concat('Participates in',
 ' project as an employee of organization.  Duties may vary.'));
insert into role values (10, 'staff participant', 'project', concat('Participates in',
 ' project as an employee of organization.  Duties may vary.'));
insert into role values (11, 'contractor', 'project', concat('Provides consulting or other',
 ' services on a contracted basis'));
insert into role values (12, 'contractor', 'system', concat('Provides consulting or other',
 ' services on a contracted basis'));
insert into role values (13, 'treasurer', 'system', concat('Maintains accounting information',
 ' records allocations and expenditures, audits accounts'));
insert into role values (14, 'treasurer', 'project space', concat('Maintains accounting information',
 ' records allocations and expenditures, audits accounts'));
insert into role values (15, 'treasurer', 'project', concat('Maintains accounting information',
 ' records allocations and expenditures, audits accounts'));
insert into role values (16, 'steering committee', 'project', concat('Participates in steering',
 ' committee to guide project as a whole'));

# Person.person_guid primary key sequence
CREATE TABLE person_guid (id INT NOT NULL);
INSERT INTO person_guid VALUES (0);

delimiter //
create procedure increment_person_guid (out current integer)
BEGIN
   update person_guid set id=last_insert_id(id+1);
   select last_insert_id() into current;
END
//
delimiter ;

# Project.person table creation
create table person (
   person_guid       integer     not null    primary key,
   name_last         varchar(32) not null,
   name_given        varchar(64) not null,
   preferred_mode    enum(
                      'e-mail',
                      'telephone')
                                 not null,
   name_first        varchar(32) default '',
   name_middle       varchar(32) default '',
   birth_date        date
) engine = InnoDB;

# project_space_privilege_guid primary key sequence
CREATE TABLE project_space_privilege_guid (id INT NOT NULL);
INSERT INTO project_space_privilege_guid VALUES (0);

delimiter //
create procedure increment_project_space_privilege_guid (out current integer)
BEGIN
   update project_space_privilege_guid set id=last_insert_id(id+1);
   select last_insert_id() into current;
END
//
delimiter ;

# project_space_privilege table creation
create table project_space_privilege (
   project_space_privilege_guid  integer     primary key,
   project_space_guid            integer     not null,
   holder_guid                   integer     not null,
   privilege_guid                integer     not null,
   is_role_bound                 boolean     default false,
   include_children              boolean     default true,
   foreign key (project_space_guid) references project_space(project_space_guid),
   foreign key (privilege_guid) references privilege(privilege_guid)
);

# Project.e_mail table creation
create table e_mail (
   person_guid   integer     not null,
   e_mail        varchar(96) not null,
   is_primary    boolean     default false,
   primary key (person_guid, e_mail),
   foreign key (person_guid) references person(person_guid)
) engine = InnoDB;

# Project.phone_number table creation
create table phone_number (
   person_guid   integer     not null,
   phone_number  char(14)    not null,
   is_primary    boolean     default false,
   primary key (person_guid, phone_number),
   foreign key (person_guid) references person(person_guid)
) engine = InnoDB;

# person_visible listing table creation
create table person_visible (
   person_guid          integer     not null,
   project_space_guid   integer     not null,
   primary key (person_guid, project_space_guid),
   foreign key (person_guid) references person (person_guid)
);
#SAMPLE:
# intitial loading from person table
#mysql> insert into person_visible select * from
#    ->    (select person_guid from person) as p_g
#    -> join
#    ->    (select 0) as default_space;
#Query OK, 9 rows affected (0.14 sec)

# Project.project_guid primary key sequence
CREATE TABLE project_guid (id INT NOT NULL);
INSERT INTO project_guid VALUES (0);

delimiter //
create procedure increment_project_guid (out current integer)
BEGIN
   update project_guid set id=last_insert_id(id+1);
   select last_insert_id() into current;
END
//
delimiter ;

# Project.project table creation
create table project (
   project_guid         integer     not null primary key,
   project_space_guid   integer     not null,
   name                 varchar(64) not null,
   description          text        not null,
   lead                 integer     not null,
   co_lead              integer,
   start_date           date,
   finish_date          date,
   foreign key (lead) references person(person_guid)
) engine = InnoDB;

# project_privilege_guid primary key sequence
CREATE TABLE project_privilege_guid (id INT NOT NULL);
INSERT INTO project_privilege_guid VALUES (0);

delimiter //
create procedure increment_project_privilege_guid (out current integer)
BEGIN
   update project_privilege_guid set id=last_insert_id(id+1);
   select last_insert_id() into current;
END
//
delimiter ;

# project_privilege table_creation
create table project_privilege (
   project_privilege_guid  integer     primary key,
   project_guid            integer     not null,
   holder_guid             integer     not null,
   privilege_guid          integer     not null,
   is_role_bound           boolean     default false,
   include_children        boolean     default true,
   foreign key (project_guid) references project(project_guid),
   foreign key (privilege_guid) references privilege(privilege_guid)
);

# Project.task table creation
create table task (
   task_guid     integer   not null primary key,
   parent_project integer   not null,
   foreign key (task_guid) references project(project_guid),
   foreign key (parent_project) references project(project_guid)
) engine = InnoDB;

# Project.participant table creation
create table participant (
   person_guid    integer        not null,
   domain_guid   integer        not null,
   role_guid      integer        not null,
   primary key  (person_guid, domain_guid, role_guid),
   foreign key (person_guid) references person(person_guid)
) engine = InnoDB;

# Expenditure.expenditure_guid primary key sequence
CREATE TABLE expenditure_guid (id INT NOT NULL);
INSERT INTO expenditure_guid VALUES (0);

delimiter //
create procedure increment_expenditure_guid (out current integer)
BEGIN
   update expenditure_guid set id=last_insert_id(id+1);
   select last_insert_id() into current;
END
//
delimiter ;

# Project.expenditure table creation
create table expenditure (
   expenditure_guid  integer        not null    primary key,
   project_guid      integer        not null,
   authorized_by     integer        not null,
   amount            decimal(15, 2) not null,
   date_incurred     date           not null,
   payee             varchar(64)    not null,
   title             varchar(64)    not null,
   description       text,
   registered        date           not null,
   foreign key  (project_guid) references project(project_guid),
   foreign key  (authorized_by) references person(person_guid)
) engine = InnoDB;

create table expenditure_change_guid (
  id integer not null
);
insert into expenditure_change_guid values (0);

# Expenditure_change.expenditure_change_guid primary key sequence
delimiter //
create procedure increment_expenditure_change_guid (out current integer)
BEGIN
   update expenditure_change_guid set id=last_insert_id(id+1);
   select last_insert_id() into current;
END
//
delimiter ;

# Project.expenditure_change table creation
create table expenditure_change (
   change_guid integer not null primary key,
   altered           date           not null,
   expenditure_guid int(11) not null,
   project_guid int  not null,
   authorized_by     integer        not null,
   amount            decimal(15, 2) not null,
   date_incurred     date           not null,
   payee             varchar(64)    not null,
   title             varchar(64)    not null,
   description       text,
   registered        date           not null,
   foreign key (expenditure_guid) references expenditure (expenditure_guid)
) engine=InnoDB;

# Allocation.allocation_guid primary key sequence
CREATE TABLE allocation_guid (id INT NOT NULL);
INSERT INTO allocation_guid VALUES (0);

delimiter //
create procedure increment_allocation_guid (out current integer)
BEGIN
   update allocation_guid set id=last_insert_id(id+1);
   select last_insert_id() into current;
END
//
delimiter ;

# Project.allocation table creation
create table allocation(
   allocation_guid   integer        not null primary key,
   project_guid      integer        not null,
   amount            decimal(15,2)  not null,
   source            varchar(64)    not null,
   date_allocated    date           not null,
   title             varchar(64)    not null,
   purpose           varchar(128)   not null,
   description       text,
   registered        date           not null,
   foreign key  (project_guid) references project(project_guid)
) engine = InnoDB;

# Allocation.allocation_change_guid primary key sequence
create table allocation_change_guid (
  id integer not null
);
insert into allocation_change_guid values (0);
delimiter //
create procedure increment_allocation_change_guid (out current integer)
BEGIN
   update allocation_change_guid set id=last_insert_id(id+1);
   select last_insert_id() into current;
END
//
delimiter ;

# Project.allocation_change table creation
create table allocation_change (
  change_guid        integer        not null primary key,
  altered            date           not null,
  allocation_guid    int(11)        not null,
  project_guid       int(11)        not null,
  amount             decimal(15,2)  not null,
  source             varchar(64)    not null,
  date_allocated     date           not null,
  title              varchar(64)    not null,
  purpose            varchar(128)   not null,
  description        text,
  registered         date           not null,
  foreign key (allocation_guid) references allocation (allocation_guid)
) engine=InnoDB;

# Milestone.milestone_guid primary key sequence
CREATE TABLE milestone_guid (id INT NOT NULL);
INSERT INTO milestone_guid VALUES (0);

delimiter //
create procedure increment_milestone_guid (out current integer)
BEGIN
   update milestone_guid set id=last_insert_id(id+1);
   select last_insert_id() into current;
END
//
delimiter ;

# Project.milestone table creation
create table milestone (
   milestone_guid    integer        not null primary key,
   project_guid      integer        not null,
   title             varchar(64)    not null,
   significance      text           not null,
   description       text           not null,
   date_goal         date,
   date_achieved     date,
   foreign key  (project_guid) references project(project_guid)
) engine = InnoDB;

# Project.deliverable_guid primary key sequence
CREATE TABLE deliverable_guid (id INT NOT NULL);
INSERT INTO deliverable_guid VALUES (0);

delimiter //
create procedure increment_deliverable_guid (out current integer)
BEGIN
   update deliverable_guid set id=last_insert_id(id+1);
   select last_insert_id() into current;
END
//
delimiter ;

# Project.deliverable table creation
create table deliverable (
   deliverable_guid  integer        not null primary key,
   project_guid      integer        not null,
   title             varchar(64)    not null,
   purpose           varchar(128)   not null,
   specification     text           not null,
   date_due          date           not null,
   date_delivered    date,
   foreign key  (project_guid) references project(project_guid)
) engine = InnoDB;

# Project.deliverable_change_guid primary key sequence
CREATE TABLE deliverable_change_guid (id INT NOT NULL);
INSERT INTO deliverable_change_guid VALUES (0);

delimiter //
create procedure increment_deliverable_change_guid (out current integer)
BEGIN
   update deliverable_change_guid set id=last_insert_id(id+1);
   select last_insert_id() into current;
END
//
delimiter ;

# Project.deliverable_change table creation
create table deliverable_change (
   change_guid       integer        not null primary key,
   altered           date           not null,
   deliverable_guid  integer        not null,
   project_guid      integer        not null,
   title             varchar(64)    not null,
   purpose           varchar(128)   not null,
   specification     text           not null,
   date_due          date           not null,
   date_delivered    date,
   registered date not null,
   foreign key  (deliverable_guid) references deliverable(deliverable_guid)
) engine = InnoDB;

# access_right_guid primary key sequence
CREATE TABLE access_right_guid (id INT NOT NULL);
INSERT INTO access_right_guid VALUES (0);

delimiter //
create procedure increment_access_right_guid (out current integer)
BEGIN
   update access_right_guid set id=last_insert_id(id+1);
   select last_insert_id() into current;
END
//
delimiter ;

create table access_right (
   access_right_guid       integer     not null primary key,
   privilege_guid          integer     not null,
   holder                  integer     not null,
   is_role                 boolean     default false,
   level                   integer     not null,
   domain_guid             integer     not null,
   subtree_included        boolean     default true
);

